import json
# style guide: you should not import explicit functions.
# Import the whole package so that in code it is clearer from which package the function comes.
# In that case:
# import flask
# and then below app = flask.Flask(__name__)
# In that case it is clear but for other functions that could be misleading

import threading
import requests
from flask import Flask, jsonify, request
from genson import SchemaBuilder
import web.webt as web
from flask_cors import CORS
import json
import pathlib
import os
import time
# Import custom python code
import logic
import schemas
from functools import wraps
from werkzeug.security import generate_password_hash, check_password_hash
import jwt
from datetime import datetime


app = Flask(__name__)
app.secret_key = "secret" # os.urandom(12)
CORS(app)

# base_route=''
base_route = '/tedias/UC2Dashboard/api/v1'
service_description = 'This is the UC2 dashboard.'
service_logs = ['initial log\r\n']
service_api_key = 'super secret secret'
service_name = 'UC2Dashboard'
service_diagnostics = {
    "uptime": "Service is up for 1 second."
}




middleware_base_address = 'http://localhost:8765/tedias/api/middleware/v1'
service_uid = ''


def authenticated(username: str, password_entered: str) -> bool:
    with open("../users") as file:
        correct_password = json.load(file)[username]
    return check_password_hash(correct_password, password_entered)

@app.route("/login", methods=["POST"])
def login():
    data = request.get_json()
    with open("../users") as file:
        usernames = list(json.load(file).keys())
    if not data or "username" not in data or "password" not in data or \
            data["username"] not in usernames or \
            not authenticated(data["username"], data["password"]):
        return {"message": "Invalid credentials", "authenticated": False}, 401
    token = jwt.encode({
        'sub': data["username"],
        'iat': datetime.utcnow(),
        #'exp': datetime.utcnow() + timedelta(minutes=180)
    }, app.secret_key, algorithm="HS256")
    
    return jsonify({'token': token})# .decode("UTF-8")})


def token_required(f):
    @wraps(f)
    def _verify(*args, **kwargs):
        auth_headers = request.headers.get("Authorization", "").split()

        invalid_message = {
            "message": "Invalid token. Authentication required",
            "authenticated": False
        }

        if len(auth_headers) != 2:
            return invalid_message, 401

        try:
            token = auth_headers[1]
            data = jwt.decode(token, app.secret_key, algorithms=["HS256"])
            with open("../users") as file:
                userlist =list(json.load(file).keys())

            if data["sub"] not in userlist:
                # logging message would be nice, but logging is not set up
                return invalid_message, 401

            return f(*args, **kwargs)

        except jwt.InvalidTokenError as error:
            # logging message would be nice, but logging is not set up
            return invalid_message, 401

    return _verify


@app.route('/')
def index():
    return 'Web App with Python Flask!'


# Necessary endpoints for middleware
def append_log(msg):
    service_logs.append(msg + '\r\n')


@app.route(base_route + '/description', endpoint='get_description', methods=['GET'])
def get_description():
    global service_description
    return jsonify(service_description)


@app.route(base_route + '/diagnostics', endpoint='get_diagnostics', methods=['GET'])
def get_diagnostics():
    global service_diagnostics
    return jsonify(service_diagnostics)


@app.route(base_route + '/logs', endpoint='get_logs', methods=['GET'])
def get_logs():
    global service_logs
    return jsonify(service_logs)


# Custom endpoints
@app.route(base_route + '/incoming-data', endpoint='incoming_data',  methods=['POST'])
def incoming_data():
    # Create Class to compare with questionnaire map
    questionnaire_logic = logic.QuestionnaireFlagCheck(questionnaire_map
                                                       ="../questionnaire_templates/Processing_Template_Neuro.json")
    # get questionnaire response from server
    # if inputs is None:
    questionnaire_response_dict = request.get_json()
    questionnaire_id = questionnaire_response_dict['id']

    # Write received questionnaire to file
    try:
        os.makedirs("../unprocessed_files")
    except FileExistsError:
        pass
    with open('../unprocessed_files/{0}.json'.format(questionnaire_id), 'w') as outfile:
        outfile.write(json.dumps(questionnaire_response_dict))

    # Process received questionnaire
    try:
        questionnaire_response_processed_dict = questionnaire_logic.process(path='../unprocessed_files/{0}.json'
                                                                            .format(questionnaire_id))
    except KeyError:
        raise KeyError("Received questionnaire does not match expected form. "
                       "Please compare questionnaire response with the loaded template. ")

    # Print result to file if
    try:
        os.makedirs("../processed_files")
    except FileExistsError:
        pass
    if 'questionnaireresponseid' in questionnaire_response_processed_dict:
        questionnaire_response_processed_str = json.dumps(questionnaire_response_processed_dict)
        with open('../processed_files/{0}.json'.format(questionnaire_id), 'w') as outfile:
            outfile.write(questionnaire_response_processed_str)
        return questionnaire_response_processed_str


# SHOULD BE GET AND NOT POST
@app.route(base_route + '/return-data', endpoint='return_data', methods=['POST'])
def return_data():
    with open('../processed_files/{0}.json'.format(request.get_json()['questionnaire_id']), 'rb') as infile:
        data_dict = json.load(infile)
    return data_dict


@app.route(base_route + '/dashboard', endpoint='dashboard', methods=['GET'])
def dashboard():
    patient_list = []
    path = '../processed_files/'
    path_list = pathlib.Path(path).glob('**/*.json')
    for path in path_list:
        patient_list.append(logic.load_questionnaire(path=path))

    return web.build_html(patient_list)


# SHOULD BE A DELETE NOT A POST
@app.route(base_route + '/remove-questionnaire', endpoint='remove_questionnaire', methods=['POST'])
def remove_questionnaire():
    questionnaire_id = request.get_json()['questionnaire_id']
    questionnaire_raw_path = '{0}/{1}.json'.format('../processed_files/', questionnaire_id)
    questionnaire_processed_path = '{0}/{1}.json'.format('../unprocessed_files/', questionnaire_id)
    if os.path.exists(questionnaire_raw_path) and os.path.exists(questionnaire_processed_path):
        os.remove(questionnaire_raw_path)
        os.remove(questionnaire_processed_path)
        return '', 204
    elif os.path.exists(questionnaire_raw_path):
        return 'Questionnaire has not been processed yet', 500
    if os.path.exists(questionnaire_processed_path):
        os.remove(questionnaire_processed_path)
        return '', 204
    else:
        return 'File not found', 404


@app.route(base_route + '/remove-all', endpoint='remove_all', methods=['DELETE'])
def remove_all():
    for folder in ['../processed_files/', '../unprocessed_files/']:
        path_list = pathlib.Path(folder).glob('**/*.json')
        for path in path_list:
            os.remove(path)
    return 'all .json files deleted', 204


@app.route("/ampel", endpoint='get_patient_list', methods=["GET"])
@token_required
def get_patient_list():
    patient_list = []
    path = '../processed_files/'
    path_list = pathlib.Path(path).glob('**/*.json')
    for path in path_list:
        patient = logic.load_questionnaire(path=path)

        patient["color"] = patient["Patient_Priority"]
        del patient["Patient_Priority"]
        patient["id"] = patient["questionnaireresponseid"]
        del patient["questionnaireresponseid"]
        patient["waiting_time"] = int((time.time() - patient["entrance"]) / 60)
        del patient["entrance"]
        patient["primary_concern"] = patient["prios"][0]["answer"][0]["valueString"]
        patient["answersInText"] = []
        for questionAndAnswers in patient["prios"][1:]:
            answers = map(lambda x: x["valueString"], questionAndAnswers["answer"])
            patient["answersInText"].append(questionAndAnswers["text"] + " " + "; ".join(answers))
        del patient["prios"]

        patient_list.append(patient)

    return jsonify(patient_list)


@app.route("/ampel", endpoint='delete_patient', methods=["POST"])
@token_required
def delete_patient():
    os.system("touch marker")
        
    # If necessary create the folders "deleted_files/unprocessed_files" and "deleted_files/processed_files"
    try:
        os.makedirs("..deleted_files/unprocessed_files")
    except FileExistsError:
        pass
    try:
        os.makedirs("..deleted_files/processed_files")
    except FileExistsError:
        pass

    data = request.get_json()
    id_ = data["id_"]
    done_how = data["done_how"]

    patient_list = []
    path = '../processed_files/'
    path_list = pathlib.Path(path).glob('**/*.json')
    for path in path_list:
        patient = logic.load_questionnaire(path=path)

        if patient["questionnaireresponseid"] != id_:
            patient["color"] = patient["Patient_Priority"]
            del patient["Patient_Priority"]
            patient["id"] = patient["questionnaireresponseid"]
            del patient["questionnaireresponseid"]
            patient["waiting_time"] = int((time.time() - patient["entrance"]) / 60)
            del patient["entrance"]
            patient["primary_concern"] = patient["prios"][0]["answer"][0]["valueString"]
            patient["answersInText"] = []
            for questionAndAnswers in patient["prios"][1:]:
                answers = map(lambda x: x["valueString"], questionAndAnswers["answer"])
                patient["answersInText"].append(questionAndAnswers["text"] + " " + "; ".join(answers))
            del patient["prios"]
            patient_list.append(patient)
        else:
            with open(path) as file:
                patientdata = json.load(file)
            patientdata["handled_in_which_way"] = done_how
            with open(path, "w") as file:
                json.dump(patientdata, file)
            os.rename(path, "../deleted_files/processed_files/" + id_ + ".json")
            os.rename("../unprocessed_files/" + id_ + ".json", "../deleted_files/unprocessed_files/" + id_ + ".json")

    return jsonify(patient_list)

# Make the WSGI interface available at the top level so wfastcgi can get it.
wsgi_app = app.wsgi_app


def generate_schema_string(obj):
    schema_generator = SchemaBuilder()
    schema_generator.add_object(obj)
    return schema_generator.to_json(indent=2)


def register_to_middleware():
    global service_uid
    global service_api_key
    global service_name
    global service_diagnostics

    headers = {
        'Content-type': 'application/json',
        'X-ApiKey': 't3d14s-s',
    }

    # Schema strings for TEDIAS Middleware descriptor
    error_schema_str = generate_schema_string('error')
    string_schema_str = generate_schema_string('result')
    diagnostics_schema_str = generate_schema_string(service_diagnostics)
    logs_schema_str = generate_schema_string([''])
    incoming_data_input_schema_str = generate_schema_string(schemas.incoming_data_input_schema())
    incoming_data_output_schema_str = generate_schema_string(schemas.incoming_data_output_schema())
    return_data_output_schema_str = generate_schema_string(schemas.return_data_output_schema())


    descriptor = {
        'apiKey': service_api_key,
        'serviceName': service_name,
        'baseUrl': 'http://' + str(HOST) + ':' + str(PORT),
        'methodDescriptors': [
            {
                "methodName": "description",
                "relativeRoute": base_route + "/description",
                "requestMethod": "GET",
                "argumentValueTypes": None,
                "returnValueTypes": {
                    "result": string_schema_str,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "diagnostics",
                "relativeRoute": base_route + "/diagnostics",
                "requestMethod": "GET",
                "argumentValueTypes": None,
                "returnValueTypes": {
                    "result": diagnostics_schema_str,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "logs",
                "relativeRoute": base_route + "/logs",
                "requestMethod": "GET",
                "argumentValueTypes": None,
                "returnValueTypes": {
                    "result": logs_schema_str,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "incoming-data",
                "relativeRoute": base_route + "/incoming-data",
                "requestMethod": "POST",
                "argumentValueTypes": {
                    "data": incoming_data_input_schema_str
                },
                "returnValueTypes": {
                    "result": incoming_data_output_schema_str,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "return-data",
                "relativeRoute": base_route + "/return-data",
                "requestMethod": "POST",
                "argumentValueTypes": {
                    "questionnaire_id": "string"
                },
                "returnValueTypes": {
                    "result": return_data_output_schema_str,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "dashboard",
                "relativeRoute": base_route + "/dashboard",
                "requestMethod": "GET",
                "argumentValueTypes": None,
                "returnValueTypes": {
                    "result": None,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "remove-questionnaire",
                "relativeRoute": base_route + "/remove-questionnaire",
                "requestMethod": "POST",
                "argumentValueTypes": {
                    "questionnaire_id": "string"
                },
                "returnValueTypes": {
                    "result": None,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "remove-all",
                "relativeRoute": base_route + "/remove-all",
                "requestMethod": "DELETE",
                "argumentValueTypes": None,
                "returnValueTypes": {
                    "result": None,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "ampel",
                "relativeRoute": base_route + "/ampel",
                "requestMethod": "GET",
                "argumentValueTypes": None,
                "returnValueTypes": {
                    "result": None,
                    "error": error_schema_str
                }
            },
            {
                "methodName": "ampel",
                "relativeRoute": base_route + "/ampel",
                "requestMethod": "POST",
                "argumentValueTypes": None,
                "returnValueTypes": {
                    "result": None,
                    "error": error_schema_str
                }
            },
        ]
    }
    body = json.dumps(descriptor)
    response = requests.post(middleware_base_address + '/services/register', data=body, headers=headers, verify=False)

    if response.status_code > 299:
        append_log('Something went wrong during registration')
        return

    service_uid = str(response.json())


def unregister_from_middleware():
    global service_uid

    headers = {
        'Content-type': 'application/x-www-form-urlencoded',
        'X-ApiKey': middlewareApiKey,
    }

    dateUrlEncoded = {"identifier": serviceIdentifier}

    response = requests.post(middleware_base_address + '/services/unregister', data=dateUrlEncoded, headers=headers,
                             verify=False)
    if response.status_code > 299:
        append_log('Something went wrong during unregistration')
        return

    service_uid = ''


def main_loop():
    while True:
        try:
            register_to_middleware()
            while True:
                print('> ', end='')
                key = input()
                if key == 'q':
                    break
            unregister_from_middleware()
        except requests.exceptions.ConnectionError:
            print('Middleware either not running or not found.')
        time.sleep(10)

def run_app(host, port):
    app.run(host=host, port=port, use_reloader=False, debug=True)


#  todo Im Orchestrator testen ->
#  	in def uc2_1_minimal():

#  todo Patienten_Liste: Als Map machen,
#  	sodass die Patienten direkt nach ID abgefragt werden können


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(description='Set HOST and PORT externally.')
    parser.add_argument('--HOST', metavar='HOST', required=False, help='Host url',
                        default=os.environ.get('SERVER_HOST', 'localhost'))
    parser.add_argument('--PORT', metavar='PORT', required=False, help='Port number',
                        default=int(os.environ.get('SERVER_PORT_', '8069')))
    args = parser.parse_args()

    HOST = args.HOST
    PORT = args.PORT

    print("Host: {0}".format(HOST))
    print("Port: {0}".format(PORT))
    print("Trivial dashboard at http://{0}:{1}{2}".format(HOST, PORT, base_route + '/dashboard'))

    web_client = threading.Thread(target=run_app, args=(HOST, PORT))
    web_client.start()

    main_loop = threading.Thread(target=main_loop)
    main_loop.start()
    main_loop.join()

    exit(0)
