import json
import os
import subprocess
import sys
import time


class QuestionnaireProcessor:
    def __init__(self, questionnaire_path: dict = None):
        if questionnaire_path is None:
            with open("../questionnaire_templates/Processing_Template_Neuro.json") as file:
                self.questionnaire = json.load(file)
        else:
            with open(questionnaire_path) as file:
                self.questionnaire = json.load(file)

    def loadUnprocessedPatientdata(self, path: str = None) -> dict: 
        if path is None:
            path = "../unprocessed_files/"+os.listdir("../unprocessed_files")[0]
        with open(path) as file:
            unprocessedPatientdata = json.load(file)
        return unprocessedPatientdata

    def processPatientdata(self, unprocessedPatientdata: dict) -> dict:
        patientdata = dict()
        self._createProcessedHeadder(patientdata, unprocessedPatientdata)
        self._createPrios(patientdata, unprocessedPatientdata)
        self._createPatientpriority(patientdata, unprocessedPatientdata)
        self._addEntranceTime(patientdata, unprocessedPatientdata)
        return patientdata

    def _createProcessedHeadder(self, patientdata: dict, unprocessedPatientdata: dict):
        patientdata["first_name"] = " ".join(unprocessedPatientdata["contained"][0]["name"][0]["given"])
        patientdata["last_name"] = unprocessedPatientdata["contained"][0]["name"][0]["family"]
        patientdata["questionnaireresponseid"] = self.questionnaire["id"]

    def _createPrios(self, patientdata: dict, unprocessedPatientdata: dict):
        patientdata["prios"] = []
        for question in unprocessedPatientdata["item"]:
            prio = "yellow"
            answer = []
            for a in question["answer"]:
                if a["flag"] == "red": 
                    prio = "red"
                if a["flag"] == "green" and prio != "red":
                    prio = "green"
                answer.append({"valueString": a["valueString"]})
            patientdata["prios"].append({"linkId": question["linkId"], "id": self.questionnaire["id"], "prio": prio, "text": question["text"], "answer": answer})

    def _createPatientpriority(self, patientdata: dict, unprocessedPatientdata: dict):
        patientdata["Patient_Priority"] = "yellow"
        for question in patientdata["prios"]:
            if question["prio"] == "red":
                patientdata["Patient_Priority"] = "red"
            if question["prio"] == "green" and patientdata["Patient_Priority"] != "red":
                patientdata["Patient_Priority"] = "green"

    def _addEntranceTime(self, patientdata: dict, unprocessedPatientdata: dict):
        patientdata["entrance"] = time.time()

    def saveProcessedData(self, data: dict, path: str = None):
        if path is None:
            path = "../processed_data/temporary.json"
        if not os.path.exists(path):
            subprocess.run(["touch", path])
            with open(path, "w") as file:
                json.dump(data,file)
        else:
            raise ValueError # file already exists... override?

