# Schemas for TEDIAS Middleware descriptor
# incomingData Schema (input). This is for providing the schema for the Middleware descriptor.
def incoming_data_input_schema():
    return {
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "authored": {
                "type": "string"
            },
            "contained": {
                "type": "array",
                "items": [
                    {
                        "type": "object",
                        "properties": {
                            "address": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "text": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "text"
                                        ]
                                    }
                                ]
                            },
                            "birthDate": {
                                "type": "string"
                            },
                            "gender": {
                                "type": "string"
                            },
                            "id": {
                                "type": "string"
                            },
                            "identifier": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "type": {
                                                "type": "object",
                                                "properties": {
                                                    "text": {
                                                        "type": "string"
                                                    }
                                                },
                                                "required": [
                                                    "text"
                                                ]
                                            },
                                            "value": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "type",
                                            "value"
                                        ]
                                    },
                                    {
                                        "type": "object",
                                        "properties": {
                                            "type": {
                                                "type": "object",
                                                "properties": {
                                                    "text": {
                                                        "type": "string"
                                                    }
                                                },
                                                "required": [
                                                    "text"
                                                ]
                                            },
                                            "value": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "type",
                                            "value"
                                        ]
                                    }
                                ]
                            },
                            "name": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "family": {
                                                "type": "string"
                                            },
                                            "given": {
                                                "type": "array",
                                                "items": [
                                                    {
                                                        "type": "string"
                                                    }
                                                ]
                                            }
                                        },
                                        "required": [
                                            "family",
                                            "given"
                                        ]
                                    }
                                ]
                            },
                            "resourceType": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "address",
                            "birthDate",
                            "gender",
                            "id",
                            "identifier",
                            "name",
                            "resourceType"
                        ]
                    }
                ]
            },
            "id": {
                "type": "string"
            },
            "identifier": {
                "type": "object",
                "properties": {
                    "type": {
                        "type": "object",
                        "properties": {
                            "text": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "text"
                        ]
                    },
                    "value": {
                        "type": "string"
                    }
                },
                "required": [
                    "type",
                    "value"
                ]
            },
            "item": {
                "type": "array",
                "items": [
                    {
                        "type": "object",
                        "properties": {
                            "answer": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "valueString": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "valueString"
                                        ]
                                    }
                                ]
                            },
                            "linkId": {
                                "type": "string"
                            },
                            "text": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "answer",
                            "linkId",
                            "text"
                        ]
                    },
                    {
                        "type": "object",
                        "properties": {
                            "answer": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "valueString": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "valueString"
                                        ]
                                    }
                                ]
                            },
                            "linkId": {
                                "type": "string"
                            },
                            "text": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "answer",
                            "linkId",
                            "text"
                        ]
                    }
                ]
            },
            "resourceType": {
                "type": "string"
            },
            "source": {
                "type": "object",
                "properties": {
                    "reference": {
                        "type": "string"
                    }
                },
                "required": [
                    "reference"
                ]
            },
            "status": {
                "type": "string"
            }
        },
        "required": [
            "authored",
            "contained",
            "id",
            "identifier",
            "item",
            "resourceType",
            "source",
            "status"
        ]
    }


# incomingData Schema (output)
def incoming_data_output_schema():
    return {
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "questionnaireresponseid": {
                "type": "string"
            },
            "first_name": {
                "type": "string"
            },
            "last_name": {
                "type": "string"
            },
            "prios": {
                "type": "array",
                "items": [
                    {
                        "type": "object",
                        "properties": {
                            "linkId": {
                                "type": "string"
                            },
                            "prio": {
                                "type": "string"
                            },
                            "id": {
                                "type": "string"
                            },
                            "text": {
                                "type": "string"
                            },
                            "answer": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "valueString": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "valueString"
                                        ]
                                    }
                                ]
                            }
                        },
                        "required": [
                            "linkId",
                            "prio",
                            "id",
                            "text",
                            "answer"
                        ]
                    },
                    {
                        "type": "object",
                        "properties": {
                            "linkId": {
                                "type": "string"
                            },
                            "prio": {
                                "type": "string"
                            },
                            "id": {
                                "type": "string"
                            },
                            "text": {
                                "type": "string"
                            },
                            "answer": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "valueString": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "valueString"
                                        ]
                                    }
                                ]
                            }
                        },
                        "required": [
                            "linkId",
                            "prio",
                            "id",
                            "text",
                            "answer"
                        ]
                    }
                ]
            },
            "Patient_Priority": {
                "type": "string"
            },
            "entrance": {
                "type": "number"
            }
        },
        "required": [
            "questionnaireresponseid",
            "first_name",
            "last_name",
            "prios",
            "Patient_Priority",
            "entrance"
        ]
    }


# returnData Schema (output)
def return_data_output_schema():
    return {
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "Patient_Priority": {
                "type": "string"
            },
            "entrance": {
                "type": "number"
            },
            "first_name": {
                "type": "string"
            },
            "last_name": {
                "type": "string"
            },
            "prios": {
                "type": "array",
                "items": [
                    {
                        "type": "object",
                        "properties": {
                            "answer": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "valueString": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "valueString"
                                        ]
                                    }
                                ]
                            },
                            "id": {
                                "type": "string"
                            },
                            "linkId": {
                                "type": "string"
                            },
                            "prio": {
                                "type": "string"
                            },
                            "text": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "answer",
                            "id",
                            "linkId",
                            "prio",
                            "text"
                        ]
                    },
                    {
                        "type": "object",
                        "properties": {
                            "answer": {
                                "type": "array",
                                "items": [
                                    {
                                        "type": "object",
                                        "properties": {
                                            "valueString": {
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "valueString"
                                        ]
                                    }
                                ]
                            },
                            "id": {
                                "type": "string"
                            },
                            "linkId": {
                                "type": "string"
                            },
                            "prio": {
                                "type": "string"
                            },
                            "text": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "answer",
                            "id",
                            "linkId",
                            "prio",
                            "text"
                        ]
                    }
                ]
            },
            "questionnaireresponseid": {
                "type": "string"
            }
        },
        "required": [
            "Patient_Priority",
            "entrance",
            "first_name",
            "last_name",
            "prios",
            "questionnaireresponseid"
        ]
    }
